

import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;

/**
 * APIMethod-Test-Servlet
 * @author   Hendrik Schreiber <hendriks@zapf.fb15.uni-dortmund.de>
 */
public class APIMethodTest extends HttpServlet 
{
   public void doPost(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
   {
      doGet(req, res);
   }

   public void doGet(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
   {
      req.getRemoteAddr(); 
      req.getRemoteHost();
      req.getScheme();
      req.getServerName();
      req.getServerPort();
      req.getContentType();
      req.getContentLength();
      req.getProtocol();
      
      // HTTP spezifische Methoden
      req.getAuthType();
      req.getCookies();
      req.getHeader("User-Agent");
      req.getMethod();
      req.getPathInfo();
      req.getPathTranslated();
      req.getQueryString();
      req.getRemoteUser();
      req.getRequestedSessionId();
      req.getRequestURI();
      req.getServletPath();
      req.isRequestedSessionIdFromCookie();
      req.isRequestedSessionIdFromUrl();
      req.isRequestedSessionIdValid();
      
      // nicht HTTP spezifische Methoden
      res.setContentType("text/html");
      
      // HTTPResponse spezifische Methoden 
      res.addCookie(new Cookie("Name", "Value"));
      res.setContentType("text/html");
      res.containsHeader("Content-Type");
      res.encodeRedirectUrl("http://www.tagtraum.com/");
      res.encodeUrl("index.html");
      res.setDateHeader("MyDate", System.currentTimeMillis());
      res.setHeader("MyHeader", "senseless stuff");
      res.setIntHeader("MyIntHeader", 42);
      res.setStatus(200);
      
      // ServletContext
      ServletContext theServletContext = getServletContext();
      theServletContext.getMimeType("file.txt");
      theServletContext.getRealPath("/");
      
      // schreibe ein Byte, um ServletExec 2.1
      // zufriedenzustellen. Ohne dieses Byte schliet ServletExec 2.1
      // die Verbindung nicht und schreibt auch keine Header
      // Dieser Fehler wurde von NewAtlanta erst mit Version 2.2
      // behoben, nachdem die meisten unserer Test schon gelaufen
      // waren.
      ServletOutputStream out = res.getOutputStream();
      out.write(42);
   }
}
HelloWorld-Test
package com.tagtraum.jsiege.servlets;

import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;

/**
 * HelloWorld-Servlet
 * @author   Hendrik Schreiber <hendriks@zapf.fb15.uni-dortmund.de>
 */
public class HelloWorld extends HttpServlet 
{
   /**
    * Schreibt "Hello World" mit einem Writer.
    *
    * @param   req ServletRequest-Object
    * @param   res ServletResponse-Object
    */
   public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
      // setze MIME-Typ auf "text/html"
      res.setContentType("text/html");
      // hole Writer
      PrintWriter out = res.getWriter();
      // schreibe "Hello World"
      out.println("Hello World");
   }
}
File-Test
package com.tagtraum.jsiege.servlets;

import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;

/**
 * IOTest-Servlet
 * @author   Hendrik Schreiber <hendriks@zapf.fb15.uni-dortmund.de>
 */
public class IOTest extends HttpServlet 
{

   public static final String C_SetContentLength = "setContentLength";
   
   private boolean mySetContentLength;
   
   public void init(ServletConfig config) throws ServletException
   {
      super.init(config);
      mySetContentLength = "true".equalsIgnoreCase(getInitParameter(C_SetContentLength));
   }
   
   public void doPost(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
   {
      doGet(req, res);
   }

   public void doGet(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
   {
      String theURI = req.getPathTranslated();
      File theFile = new File(theURI);
      if (mySetContentLength) {
         res.setContentLength((int)theFile.length());
      }
      res.setContentType(getServletContext().getMimeType(theURI));
      byte[] buf = new byte[4096];
      int read = 0;
      FileInputStream in = new FileInputStream(theFile);
      ServletOutputStream out = res.getOutputStream();
      try {
         while ((read = in.read(buf, 0, buf.length)) != -1)
         {
            out.write(buf, 0, read);
         }
      }
      finally {
         if (in != null) in.close();
         if (out != null) out.close();
      }
   }
}
